#include <jabber/jlib.h>
#include <jabber/etherx.h>

#define JP_UNKNOWN 0
#define JP_INCOMING 1
#define JP_DELIVER 2
#define JP_SEND 3

#define JP_FLAG_NONE 0
#define JP_FLAG_AUTHREQ 1
#define JP_FLAG_AUTHOK 2

#define CMD_SET 0
#define CMD_GET 1
#define CMD_CHECK 2
#define CMD_DEL 3

#define M_FLAG_OK 0
#define M_FLAG_IGNORE_PHASE 1
#define M_FLAG_IGNORE_SESSION 2

#define M_PHASE_NONE 0
#define M_PHASE_AUTH 1
#define M_PHASE_SESSION_ON 2
#define M_PHASE_SESSION_OFF 3
#define M_PHASE_MESSAGE_IN 4
#define M_PHASE_MESSAGE_OUT 5
#define M_PHASE_PRESENCE_IN 6
#define M_PHASE_PRESENCE_OUT 7
#define M_PHASE_IQ 8
#define M_PHASE_CONFIG 10

#define RESOURCE_SESSION 0
#define RESOURCE_REMOTE 1

char *config_names(int cmd, char *name);
char *config_transports(int cmd, char *id, xmlnode x);
void config_service(xmlnode x);
void config_startup(void);
void config_listen(xmlnode x);

void etherx_receive(epacket e, int flag);

typedef struct resource_struct
{
    int type;
    char *user;
    char *res;
    pth_msgport_t handler;
    struct resource_struct *next;
} _resource, *resource;

resource resource_create(int type, char *user, char *res, pth_msgport_t handler);
resource resource_lookup(char *user, char *res);
void resource_free(resource r);
void deliver(jpacket p);
jid remote(int cmd, char *user, char *res, jid real);

typedef void (*session_onSend)(thread t, jpacket p);
 
typedef struct session_struct
{
    resource r;
    pth_msgport_t mp_from;
    pth_msgport_t mp_to;
    session_onSend send_evt;
    void *send_evt_data;
    char *id;
    struct session_struct *next;
} *session;

thread session_create(jid owner, session_onSend evt, void *evt_data);
void *session_main(void *arg);

void conn_connect(thread t, int sock, struct sockaddr_in sa);
void conn_negotiate(thread t, xmlnode in, int type);
xmlstream_onNode conn_namespace(int cmd, char *ns, xmlstream_onNode handler);

void *remote_main(void *arg);
void *server_main(void *arg);
void *external_main(void *arg);
void *unknown_main(void *arg);

typedef struct {
    pth_message_t head; /* the standard pth message header */
    jpacket p;
} _jpq, *jpq;

void psend(pth_msgport_t mp, jpacket p); /* sends p to a pth message port */
void *dso_load(char *so, char *init, void *arg);

typedef struct mapi_struct
{
    int phase;
    session s;
    jpacket pkt;
    pool p;
    /* other objects passed to modules for all dispatch calls */
} mapi;

void module_dispatch(int phase, session s, jpacket p);
