#include "jserver.h"

struct _svc_basic_connections
{
    session s;
    thread t;
    struct _svc_basic_connections *next;
};

/* internal lookup routine, associating the connection thread and it's session */
session _svc_basic_lookup(int cmd, thread t, session s)
{
    static struct _svc_basic_connections *l = NULL;
    struct _svc_basic_connections *next, *prev;
 
    if(t == NULL)
        return NULL;
 
    switch(cmd)
    {
        case CMD_SET: /* saves this thread/session pair */
            if(_svc_basic_lookup(CMD_CHECK,t,NULL) != NULL)
                _svc_basic_lookup(CMD_DEL,t,NULL); /* make sure we remove any old associations */

            next = malloc(sizeof(struct _svc_basic_connections));
            next->s = s;
            next->t = t;
            next->next = l;
            l = next;
            return s;
            break;
        case CMD_CHECK: /* returns the session for the given thread or returns NULL */
            next = l;
            while(next != NULL)
            {
                if(t == next->t)
                    return next->s;
                next = next->next;
            }
            break;
        case CMD_DEL: /* removes this thread/session association */
            prev = next = l;
            while(next != NULL)
            {
                if(t == next->t)
                {
                    if(next == l)
                        l = next->next;
                    else
                        prev->next = next->next;
                    free(next);
                    return NULL;
                }
                prev = next;
                next = next->next;
            }
            break;
    }

    return NULL;
}

void svc_basic_onSend(thread t, jpacket p)
{
    session s = (session)(t->data);
    thread t_socket = (thread)(s->send_evt_data);
    xmlnode from;

    xmlnode_hide_child(p->x, xmlnode_get_tag(p->x, "to"));
    if(p->from != NULL)
    {
        xmlnode_hide_child(p->x, xmlnode_get_tag(p->x, "from"));
        from = xmlnode_insert_tag(p->x, "from");
        xmlnode_insert_cdata(from, jid_full(p->from), -1);
    }

    printf("got[%d] a packet[%d] to send to the user[%d]",s,p,t_socket);

    xmlstream_write(t_socket, p->x);
}

/* the connection/stream has passed all the test by the server when it get's here */
void svc_basic_onNode(thread t, xmlnode in, int type)
{
    tstream ts = (tstream)(t->data);
    session s;
    thread tnew;
    xmlnode in2, from;
    jpacket p;
    static int key = 1;
    char *to;

    printf("jserver basic XML NODE: %d type[%d] [%d]'%s'\n",ts->fd,type,in,xmlnode2str(in));
 
    s = _svc_basic_lookup(CMD_CHECK, t, NULL);
    
    if(type == XMLSTREAM_ROOT)
    {
        xmlstream_write_header(t, xmlstream_header("jabber:client",NULL,xmlnode_get_attrib(in, "to")));
        /* XXX we need to keep this somewhere, to know what server name they use (the to header) */
        to = pmalloc_key(t->p, (int)&key, strlen(xmlnode_get_attrib(in, "to")) + 1);
        memcpy(to, xmlnode_get_attrib(in, "to"), strlen(xmlnode_get_attrib(in, "to")) + 1);
        return;
    }

    if(type == XMLSTREAM_CLOSE)
    {
        /* the connection was closed */
        printf("connection closed\n");
        return;
    }

    if(type == XMLSTREAM_ERROR)
    {
        /* the connection was closed */
        printf("XML error, connection closed\n");
        xmlstream_error(t, "Invalid XML Received");
        return;
    }

    if(s == NULL)
    { /* no session yet, look for auth */
        if(j_strcmp(xmlnode_get_name(in), "iq") == 0 && j_strcmp(xmlnode_get_attrib(xmlnode_get_tag(in, "query"),"xmlns"),"jabber:iq:auth") == 0)
        {
            p = jpacket_new(xmlnode_dup(in));
            to = pmalloc_key(t->p, (int)&key, 0);
            debug(to);
            p->from = jid_new(p->p, to);
            module_dispatch(M_PHASE_AUTH, NULL, p);
            if(p->flag == JP_FLAG_AUTHOK)
            {
                if(resource_lookup(p->from->user, p->from->resource) != NULL)
                {
                    /* XXX there's an existing resource, can't have two! throw error */
                }
                printf("creating new session for %s\n",jid_full(p->from));
                tnew = session_create(p->from, svc_basic_onSend, (void *)t);
                if(tnew != NULL)
                {
                    s = (session)(tnew->data);
                    _svc_basic_lookup(CMD_SET, t, s);
                }
            }
            xmlstream_write(t, p->x);
        }else{
            debug("no session found for this connection!");
        }
    }else{
        debug("sending packet to session");
        p = jpacket_new(xmlnode_dup(in));
        p->from = jid_new(p->p, s->id);
        psend(s->mp_from, p);
    }
}


