#include "jserver.h"

thread session_create(jid owner, session_onSend evt, void *evt_data)
{
    /* creats a new session, registers the resource for it, and spawns a thread to handle it */
    thread t;
    session s;

    if(owner == NULL || owner->resource == NULL) /* illegal */
        return NULL;
 
    log(LOG_DEBUG,"session_create",NULL);
 
    t = thread_new();
    s = pmalloc(t->p, sizeof(struct session_struct));
    s->mp_from = pth_msgport_create("session_from");
    s->mp_to = pth_msgport_create("session_to");
    s->id = pstrdup(t->p, jid_full(owner));
    s->r = resource_create(RESOURCE_SESSION, owner->user, owner->resource, s->mp_to);
printf("s[%d]s->r[%d]\n",s, s->r);
    s->send_evt = evt;
    s->send_evt_data = evt_data;
    t->data = (void *)s;
    t->id = pth_spawn(PTH_ATTR_DEFAULT, session_main, (void *)t);
    pth_yield(NULL); /* let it have a chance to start up, XXX NULL should be t->id but it seems to be broken in pth */
 
    return t;
}


void *session_main(void *arg)
{
    thread t = (thread)arg;
    session s = (session)(t->data);
    pth_event_t from, to, ring;
    jpq q;
    
    debug("THREAD:SESSION starting");

    from = pth_event(PTH_EVENT_MSG,s->mp_from);
    to = pth_event(PTH_EVENT_MSG,s->mp_to);
    ring = pth_event_concat(from, to, NULL);

    module_dispatch(M_PHASE_SESSION_ON,s,NULL);

    while(1)
    {
        pth_wait(ring);

        while(1)
        {
            /* this loop looks for messages from our user that need to be delivered elsewhere */
            q = (jpq)pth_msgport_get(s->mp_from);
            if(q == NULL)
                break;

            debug("THREAD:SESSION:FROM received a message!");

printf("s[%d]s->r[%d]\n",s, s->r);
            if(s->r->user != NULL)
            {
                switch(q->p->type)
                {
                    case JPACKET_MESSAGE:
                        module_dispatch(M_PHASE_MESSAGE_OUT,s,q->p);
                        break;
                    case JPACKET_PRESENCE:
                        module_dispatch(M_PHASE_PRESENCE_OUT,s,q->p);
                        break;
                }
            }

            /* if this is a close packet, exit the session */

            /* pass these to the general delivery function */
            deliver(q->p);
        }

        while(1)
        {
            /* this loop looks for messages to our user that need to be sent to them */
            q = (jpq)pth_msgport_get(s->mp_to);
            if(q == NULL)
                break;

            debug("THREAD:SESSION:TO received a message!");

            if(s->r->user != NULL)
            {
                switch(q->p->type)
                {
                    case JPACKET_MESSAGE:
                        module_dispatch(M_PHASE_MESSAGE_IN,s,q->p);
                        break;
                    case JPACKET_PRESENCE:
                        module_dispatch(M_PHASE_PRESENCE_IN,s,q->p);
                        break;
                    case JPACKET_IQ:
                        module_dispatch(M_PHASE_IQ,s,q->p);
                        break;
                }
            }

            /* deliver outgoing for this session to the onSend event passed when this session was created */
            (s->send_evt)(t, q->p);
        }
    }
    
    module_dispatch(M_PHASE_SESSION_OFF,s,NULL);

    pth_event_free(ring,PTH_FREE_ALL);
    pth_msgport_destroy(s->mp_from);
    pth_msgport_destroy(s->mp_to);
}

